<?php
/**
 * describe one specific day
 *
 * @todo add a parameter to provide a calendar view (current month + links to next month)
 *
 * This overlay accepts following parameters:
 *
 * - without_past_dates - When this parameter is provided, only future dates are listed.
 * When the parameter is absent, all dates are listed, even past dates.
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */
class Day extends Overlay {

	/**
	 * get form fields to change the day
	 *
	 * @see overlays/overlay.php
	 *
	 * @param the hosting attributes
	 * @return a list of ($label, $input, $hint)
	 */
	function get_fields($host) {
		global $context;

		// the target day
		$label = i18n::s('Day');
		$input = Skin::build_input('date_stamp', isset($this->attributes['date_stamp'])?$this->attributes['date_stamp']:'', 'date');
		$hint = i18n::s('Use format YYYY-MM-DD');
		$fields[] = array($label, $input, $hint);

		return $fields;
	}

	/**
	 * identify one instance
	 *
	 * This function returns a string that identify uniquely one overlay instance.
	 * When this information is saved, it can be used later on to retrieve one page
	 * and its content.
	 *
	 * @returns a unique string, or NULL
	 *
	 * @see articles/edit.php
	 */
	function get_id() {
		return 'day:'.$this->attributes['date_stamp'];
	}

	/**
	 * get an overlaid label
	 *
	 * Accepted action codes:
	 * - 'edit' the modification of an existing object
	 * - 'delete' the deleting form
	 * - 'new' the creation of a new object
	 * - 'view' a displayed object
	 *
	 * @see overlays/overlay.php
	 *
	 * @param string the target label
	 * @param string the on-going action
	 * @return the label to use
	 */
	function get_label($name, $action='view') {
		global $context;

		// the target label
		switch($name) {

		// description label
// 		case 'description':
// 			return i18n::s('Day description');

		// description hint
		case 'description_hint':
			return i18n::s('You should attach any relevant file or link for this day');

		// page title
		case 'page_title':

			switch($action) {

			case 'edit':
				return i18n::s('Edit one day');

			case 'delete':
				return i18n::s('Delete one day');

			case 'new':
				return i18n::s('New day');

			case 'view':
			default:
				// use article title as the page title
				return NULL;

			}
		}

		// no match
		return NULL;
	}

	/**
	 * display the content of one instance
	 *
	 * Accepted variant codes:
	 * - 'view' - embedded into the main viewing page
	 *
	 * @see overlays/overlay.php
	 *
	 * @param string the on-going action
	 * @param array the hosting record
	 * @return some HTML to be inserted into the resulting page
	 */
	function get_text($variant='view', $host=NULL) {
		global $context;

		// add something only to zooming views
		if($variant != 'view')
			return '';

		// the date
		return '<p class="day">'.Skin::build_date($this->attributes['date_stamp'], 'day')."</p>\n";
	}

	/**
	 * retrieve the content of one modified overlay
	 *
	 * @see overlays/overlay.php
	 *
	 * @param the fields as filled by the end user
	 * @return the updated fields
	 */
	function parse_fields($fields) {

		$this->attributes['date_stamp'] = $fields['date_stamp'];

		return $this->attributes;
	}

	/**
	 * remember an action once it's done
	 *
	 * To be overloaded into derivated class
	 *
	 * @param string the action 'insert', 'update' or 'delete'
	 * @param array the hosting record
	 * @return FALSE on error, TRUE otherwise
	 */
	function remember($variant, $host) {
		global $context;

		// remember the id of the master record
		$id = $host['id'];

		// set default values for this editor
		$this->attributes = Surfer::check_default_editor($this->attributes);

		// we use the existing back-end for dates
		include_once $context['path_to_root'].'dates/dates.php';

		// build the update query
		switch($variant) {

		case 'delete':

			// delete dates for this anchor
			Dates::delete_for_anchor('article:'.$id);
			break;

		case 'insert':

			// bind one date to this record
			$fields = array();
			$fields['anchor'] = 'article:'.$id;
			$fields['date_stamp'] = $this->attributes['date_stamp'];

			// update the database
			if(!$id = Dates::post($fields)) {
				Skin::error(sprintf(i18n::s('Impossible to create date %s'), $this->attributes['date_stamp']));
				return FALSE;
			}
			break;

		case 'update':

			// bind one date to this record
			$fields = array();
			$fields['anchor'] = 'article:'.$id;
			$fields['date_stamp'] = $this->attributes['date_stamp'];

			// there is an existing record
			if($date =& Dates::get_for_anchor('article:'.$id)) {

				// update the record
				$fields['id'] = $date['id'];
				if(!$id = Dates::post($fields)) {
					Skin::error(sprintf(i18n::s('Impossible to update date %s'), $this->attributes['date_stamp']));
					return FALSE;
				}

			// create a record instead of raising an error, we are smart y'a'know
			} else {
				if(!$id = Dates::post($fields)) {
					Skin::error(sprintf(i18n::s('Impossible to create date %s'), $this->attributes['date_stamp']));
					return FALSE;
				}
			}
			break;
		}

		return TRUE;
	}

	/**
	 * list dates at some section
	 *
	 * @todo in monthly_view, ensure we are listing previous or next months as well
	 *
	 * @param string the anchor to consider (e.g., 'section:123')
	 * @param int page index
	 * @return string to be inserted in resulting web page
	 */
	function render_articles_for_anchor($anchor, $page=1) {
		global $context;

		// text to be embedded in the resulting page
		$text = '';

		// we will build a list of dates
		include_once $context['path_to_root'].'dates/dates.php';

		// the maximum number of articles per page
		if(!defined('DATES_PER_PAGE'))
			define('DATES_PER_PAGE', 50);

		// should we display all dates, or not?
		$without_past_dates = FALSE;
		if(preg_match('/\bwithout_past_dates\b/i', $this->attributes['overlay_parameters']))
			$without_past_dates = TRUE;

		// on first page display future dates
		if(($page <= 1) && $without_past_dates) {

			if($items = Dates::list_future_for_anchor($anchor, 0, 100, 'family'))
				$text .= BR.BR.$items;

		}

		// where we are
		$offset = ($page - 1) * DATES_PER_PAGE;

		// deliver a calendar view for current month
		if(preg_match('/\bmonthly_view\b/i', $this->attributes['overlay_parameters']))
			$text .= Dates::build_month_calendar(gmstrftime('%Y'), gmstrftime('%m'), 'month', $anchor);

		// list all dates
		elseif(!$without_past_dates && ($items = Dates::list_for_anchor($anchor, $offset, DATES_PER_PAGE, 'family'))) {

			// navigation bar
			$bar = array();

			// count the number of dates in this section
			$stats = Dates::stat_for_anchor($anchor);
			if($stats['count'] > DATES_PER_PAGE)
				$bar = array_merge($bar, array('_count' => sprintf(i18n::ns('1&nbsp;date', '%d&nbsp;dates', $stats['count']), $stats['count'])));

			// navigation commands for dates
			include_once $context['path_to_root'].'sections/sections.php';
			$home = Sections::get_url(str_replace('section:', '', $anchor));
			$prefix = Sections::get_url(str_replace('section:', '', $anchor), 'navigate', 'articles');
			$bar = array_merge($bar, Skin::navigate($home, $prefix, $stats['count'], DATES_PER_PAGE, $page));

			// display the bar
			if(@count($bar))
				$text = Skin::build_list($bar, 'menu_bar');

			// display dates
			$text .= $items;

		// list past dates, but only to associates
		} elseif(Surfer::is_empowered() && ($items = Dates::list_past_for_anchor($anchor, $offset, DATES_PER_PAGE, 'compact'))) {

			if(@count($items))
				$items =& Skin::build_list($items, 'compact');

			// navigation bar
			$bar = array();

			// count the number of dates in this section
			$stats = Dates::stat_past_for_anchor($anchor);
			if($stats['count'] > DATES_PER_PAGE)
				$bar = array_merge($bar, array('_count' => sprintf(i18n::ns('1&nbsp;date', '%d&nbsp;dates', $stats['count']), $stats['count'])));

			// navigation commands for dates
			include_once $context['path_to_root'].'sections/sections.php';
			$home = Sections::get_url(str_replace('section:', '', $anchor));
			$prefix = Sections::get_url(str_replace('section:', '', $anchor), 'navigate', 'articles');
			$bar = array_merge($bar, Skin::navigate($home, $prefix, $stats['count'], DATES_PER_PAGE, $page));

			// display the bar
			if(@count($bar))
				$items = Skin::build_list($bar, 'menu_bar').$items;

			$text .= Skin::build_box(i18n::s('Previous dates'), $items, 'section', 'past_dates');
		}

		return $text;
	}
}

?>