<?php
/**
 * layout sections
 *
 * This is the default layout for sections.
 *
 * @see sections/index.php
 * @see sections/sections.php
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @author GnapZ
 * @tester Christophe Battarel [email]christophe.battarel@altairis.fr[/email]
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */
Class Layout_sections extends Layout_interface {

	/**
	 * list sections
	 *
	 * Accept following variants:
	 * - 'full' - include anchor information -- also the default value
	 * - 'references' - like 'full', but urls are references to sections
	 * - 'select' - like 'full', but urls are links to the article editor form - used at articles/edit.php
	 *
	 * @param resource the SQL result
	 * @return string the rendered text
	 *
	 * @see skins/layout.php
	**/
	function &layout(&$result) {
		global $context;

		// empty list
		if(!SQL::count($result)) {
			$output = array();
			return $output;
		}

		// sanity check
		if(!isset($this->layout_variant))
			$this->layout_variant = 'full';

		// load localized strings
		i18n::bind('sections');

		// flag sections updated recently
		if($context['site_revisit_after'] < 1)
			$context['site_revisit_after'] = 2;
		$dead_line = gmstrftime('%Y-%m-%d %H:%M:%S', mktime(0,0,0,date("m"),date("d")-$context['site_revisit_after'],date("Y")));
		$now = gmstrftime('%Y-%m-%d %H:%M:%S');

		// we return an array of ($url => $attributes)
		$items = array();

		// process all items in the list
		while($item =& SQL::fetch($result)) {

			// the url to view this item
			if($this->layout_variant == 'references')
				$url = 'section:'.$item['id'];
			elseif($this->layout_variant == 'select')
				$url = 'articles/edit.php?anchor='.urlencode('section:'.$item['id']);
			else
				$url = Sections::get_url($item['id'], 'view', $item['nick_name']);

			// reset the rendering engine between items
			if(is_callable(array('Codes', 'initialize')))
				Codes::initialize($url);

			// initialize variables
			$prefix = $label = $suffix = $icon = '';

			// not too many details on mobiles
			if($this->layout_variant != 'mobile') {

				// flag sections that are draft, dead, or created or updated very recently
				if($item['activation_date'] >= $now)
					$prefix .= DRAFT_FLAG;
				elseif(($item['expiry_date'] > NULL_DATE) && ($item['expiry_date'] <= $now))
					$prefix .= EXPIRED_FLAG;
				elseif($item['create_date'] >= $dead_line)
					$suffix .= NEW_FLAG;
				elseif($item['edit_date'] >= $dead_line)
					$suffix .= UPDATED_FLAG;

			}

			// signal restricted and private sections
			if($item['active'] == 'N')
				$prefix .= PRIVATE_FLAG;
			elseif($item['active'] == 'R')
				$prefix .= RESTRICTED_FLAG;

			// the introductory text
			if($item['introduction'])
				$suffix .= ' -&nbsp;'.Codes::beautify($item['introduction'], $item['options']);

			// details
			$details = array();

			// count related sub-elements
			$related_count = 0;

			// info on related sections
			$stats = Sections::stat_for_anchor('section:'.$item['id']);
			if($stats['count'])
				$details[] = sprintf(i18n::ns('1&nbsp;section', '%d&nbsp;sections', $stats['count']), $stats['count']);
			$related_count += $stats['count'];

			// info on related articles
			include_once $context['path_to_root'].'articles/articles.php';
			$stats = Articles::stat_for_anchor('section:'.$item['id']);
			if($stats['count'])
				$details[] = sprintf(i18n::ns('1&nbsp;page', '%d&nbsp;pages', $stats['count']), $stats['count']);
			$related_count += $stats['count'];

			// info on related files
			include_once $context['path_to_root'].'files/files.php';
			$stats = Files::stat_for_anchor('section:'.$item['id']);
			if($stats['count'])
				$details[] = sprintf(i18n::ns('1&nbsp;file', '%d&nbsp;files', $stats['count']), $stats['count']);
			$related_count += $stats['count'];

			// info on related links
			include_once $context['path_to_root'].'links/links.php';
			$stats = Links::stat_for_anchor('section:'.$item['id']);
			if($stats['count'])
				$details[] = sprintf(i18n::ns('1&nbsp;link', '%d&nbsp;links', $stats['count']), $stats['count']);
			$related_count += $stats['count'];

			// info on related comments
			include_once $context['path_to_root'].'comments/comments.php';
			$stats = Comments::stat_for_anchor('section:'.$item['id']);
			if($stats['count'])
				$details[] = sprintf(i18n::ns('1&nbsp;comment', '%d&nbsp;comments', $stats['count']), $stats['count']);

			// append details to the suffix
			if(count($details))
				$suffix .= ' <span class="details">('.implode(', ', $details).')</span>';

			// not if decorated
			if(($this->layout_variant != 'decorated') && ($this->layout_variant != 'references')) {

				// add sub-sections
				$details = array();
				include_once $context['path_to_root'].'sections/sections.php';
				if($related =& Sections::list_by_title_for_anchor('section:'.$item['id'], 0, YAHOO_LIST_SIZE, 'compact')) {
					foreach($related as $link => $label) {
						$sub_prefix = $sub_suffix = $sub_hover = '';
						if(is_array($label)) {
							$sub_prefix = $label[0];
							$sub_suffix = $label[2];
							if(@$label[5])
								$sub_hover = $label[5];
							$label = $label[1];
						}
						$details[$link] = array($sub_prefix, $label, $sub_suffix, 'basic', NULL, $sub_hover);
					}
				}

				// add related articles if necessary
				include_once $context['path_to_root'].'articles/articles.php';
				if(preg_match('/\barticles_by_title\b/i', $item['options']))
					$order = 'title';
				elseif(preg_match('/\barticles_by_publication\b/i', $item['options']))
					$order = 'publication';
				elseif(preg_match('/\barticles_by_rating\b/i', $item['options']))
					$order = 'rating';
				else
					$order = 'date';
				if((count($details) < YAHOO_LIST_SIZE) && ($related =& Articles::list_for_anchor_by($order, 'section:'.$item['id'], 0, YAHOO_LIST_SIZE - count($details), 'compact'))) {
					foreach($related as $link => $label) {
						$sub_prefix = $sub_suffix = $sub_hover = '';
						if(is_array($label)) {
							$sub_prefix = $label[0];
							$sub_suffix = $label[2];
							if(@$label[5])
								$sub_hover = $label[5];
							$label = $label[1];
						}
						$details[$link] = array($sub_prefix, $label, $sub_suffix, 'basic', NULL, $sub_hover);
					}
				}

				// add related files if necessary
				include_once $context['path_to_root'].'files/files.php';
				if((count($details) < YAHOO_LIST_SIZE) && ($related = Files::list_by_date_for_anchor('section:'.$item['id'], 0, YAHOO_LIST_SIZE - count($details), 'compact'))) {
					foreach($related as $link => $label) {
						$sub_prefix = $sub_suffix = $sub_hover = '';
						if(is_array($label)) {
							$sub_prefix = $label[0];
							$sub_suffix = $label[2];
							if(@$label[5])
								$sub_hover = $label[5];
							$label = $label[1];
						}
						$details[$link] = array($sub_prefix, $label, $sub_suffix, 'basic', NULL, $sub_hover);
					}
				}

				// add related links if necessary
				include_once $context['path_to_root'].'links/links.php';
				if((count($details) < YAHOO_LIST_SIZE) && ($related = Links::list_by_date_for_anchor('section:'.$item['id'], 0, YAHOO_LIST_SIZE - count($details), 'compact'))) {
					foreach($related as $link => $label) {
						$sub_prefix = $sub_suffix = $sub_hover = '';
						if(is_array($label)) {
							$sub_prefix = $label[0];
							$sub_suffix = $label[2];
							if(@$label[5])
								$sub_hover = $label[5];
							$label = $label[1];
						}
						$details[$link] = array($sub_prefix, $label, $sub_suffix, 'basic', NULL, $sub_hover);
					}
				}

				// give me more
				if(count($details) && ($related_count > YAHOO_LIST_SIZE))
					$details[Sections::get_url($item['id'], 'view', $item['nick_name'])] = array(NULL, i18n::s('More').MORE_IMG, NULL, 'more', NULL, i18n::s('This section contains other information of interest'));

				// one line per related item
				if(count($details))
					$suffix .= '<span class="details">'.Skin::build_list($details, 'compact')."</span>\n";

			}

			// start the label with family, if any
			$label = '';
			if($item['family'])
				$label = ucfirst(Codes::beautify(strip_tags($item['family'], '<br><div><img><p><span>'))).' - ';

			// use the title to label the link
			$label .= ucfirst(Codes::beautify(strip_tags($item['title'], '<br><div><img><p><span>')));

			// the icon to put in the left column
			if($item['thumbnail_url'])
				$icon = $item['thumbnail_url'];

			// some hovering title for this section
			$hover = i18n::s('View the main page of this section');

			// list all components for this item
			$items[$url] = array($prefix, $label, $suffix, 'section', $icon, $hover);

		}

		// end of processing
		SQL::free($result);
		return $items;
	}

}

?>