<?php
/**
 * the library to send electronic messages
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */

class Mailer {

	/**
	 * send an email message
	 *
	 * This function supports automated posts of e-mail message to back-office administrators.
	 *
	 * First of all, e-mail services have to be explicitly activated in the main configuration panel,
	 * at [script]control/configure.php[/script].
	 *
	 * Second, this function is able to authenticate to the mail server using POP3 before engaging SMTP.
	 * This is the standard method used at ovh to avoid spam.
	 *
	 * Third, long lines of the message are wrapped according to [link=Dan's suggestion]http://mailformat.dan.info/body/linelength.html[/link].
	 *
	 * @link http://mailformat.dan.info/body/linelength.html Dan's Mail Format Site: Body: Line Length
	 *
	 * Messages are sent using utf-8, and are base64-encoded.
	 *
	 * @link http://www.sitepoint.com/article/advanced-email-php/3 Advanced email in PHP
	 *
	 * @param string recipient address
	 * @param string subject
	 * @param string actual message
	 * @return an error message, if any
	 *
	 * @see agents/messages.php
	 * @see control/configure.php
	 * @see query.php
	 * @see shared/logger.php
	 * @see users/users.php
	 */
	function notify($to, $subject, $message) {
		global $context;

		// email services have to be activated
		if(!isset($context['with_email']) || ($context['with_email'] != 'Y'))
			return '';

		// ensure we have a sender
		if(isset($context['mail_from']) && $context['mail_from'])
			$from = $context['mail_from'];
		else
			$from = 'yacs at '.$context['host_name'];

		// do the job
		return Mailer::post($from, $to, $subject, $message);
	}

	/**
	 * send an email message
	 *
	 * @todo allow for multipart messages http://www.php.net/mail
	 *
	 * This function supports automated posts of e-mail message.
	 *
	 * For this to work, e-mail services have to be explicitly activated in the
	 * main configuration panel, at [script]control/configure.php[/script].
	 *
	 * This function is able to authenticate to the mail server using POP3
	 * before engaging SMTP. This is the standard method used at many ISPs to
	 * to avoid spam.
	 *
	 * Several recipients can be provided as a list of addresses separated by
	 * commas.
	 * For bulk posts, recipients can be transmitted as an array of strings.
	 * In all cases, this function send separately one message per recipient.
	 *
	 * Bracketed recipients, such as ##Foo Bar <foo@bar.com>##, are handled properly,
	 * meaning ##foo@bar.com## is transmitted to the mailing function, while
	 * the string ##To : Foo Bar <foo@bar.com>## is added to headers.
	 *
	 * Long lines of the message are wrapped according to [link=Dan's suggestion]http://mailformat.dan.info/body/linelength.html[/link].
	 *
	 * @link http://mailformat.dan.info/body/linelength.html Dan's Mail Format Site: Body: Line Length
	 *
	 * Messages are sent using utf-8, and are base64-encoded.
	 *
	 * @link http://www.sitepoint.com/article/advanced-email-php/3 Advanced email in PHP
	 *
	 * This function returns the number of successful posts,
	 * and populates the error context, where applicable.
	 *
	 * @param string sender address
	 * @param mixed recipient address(es)
	 * @param string subject
	 * @param string actual message
	 * @param mixed additional headers, if any
	 * @param string the originating script, if any
	 * @return the number of actual posts, or 0
	 *
	 * @see articles/mail.php
	 * @see letters/new.php
	 * @see users/mail.php
	 */
	function post($from, $to, $subject, $message, $headers='', $debug='shared/mailer.php') {
		global $context;

		// email services have to be activated
		if(!isset($context['with_email']) || ($context['with_email'] != 'Y')) {
			Skin::error(i18n::s('E-mail has not been enabled on this system.'));
			return 0;

		// email services are not allowed
		} elseif(!is_callable('mail')) {
			Skin::error(i18n::s('E-mail is not authorized on this system.'));
			return 0;

		// check sender address
		} elseif(!$from) {
			Skin::error(i18n::s('Empty sender address'));
			return 0;

		// check recipient address
		} elseif(!$to) {
			Skin::error(i18n::s('Empty recipient address'));
			return 0;

		// check mail subject
		} elseif(!$subject) {
			Skin::error(i18n::s('No subject'));
			return 0;

		// check mail content
		} elseif(!$message) {
			Skin::error(i18n::s('No message'));
			return 0;
		}

		// authenticate to a pop3 server if necessary
		if(!isset($context['mail_pop3_handle'])
			&& isset($context['mail_pop3_server']) && isset($context['mail_pop3_user']) && isset($context['mail_pop3_password'])
			&& is_callable('imap_open'))
			$context['mail_pop3_handle'] = @imap_open('{'.$context['mail_pop3_server'].':110/pop3}INBOX', $context['mail_pop3_user'], $context['mail_pop3_password']);

		// set the SMTP server
		if(isset($context['mail_smtp_server']) && $context['mail_smtp_server'])
			Safe::ini_set('SMTP', $context['mail_smtp_server']);

		// set the SMTP sender
		if(isset($context['mail_from']) && $context['mail_from'])
			Safe::ini_set('sendmail_from', $context['mail_from']);

		// no new line nor HTML tag in title
		if(!$subject)
			$subject = '***';
		$subject = preg_replace('/\s+/', ' ', strip_tags($subject));

		// make it utf-8
		$subject = utf8::from_unicode($subject);

		// encode it for the transfer
		$subject = '=?utf-8?B?'.base64_encode($subject).'?=';

		// Unix-style newlines only
		$message = str_replace("\r\n", "\n", $message);

		// wrapping threshold
		if(!defined('WRAPPING_LENGTH'))
			define('WRAPPING_LENGTH', 70);

		// wrap the message if necessary
		$lines = explode("\n", $message);
		$message = '';
		foreach($lines as $line)
			$message .= wordwrap($line, WRAPPING_LENGTH, " \n", 0)."\n";

		// make some text out of an array
		if(is_array($headers))
			$headers = implode("\n", $headers);

		// From: header
		if(!preg_match('/^From: /im', $headers))
			$headers .= "\n".'From: '.$from;

		// Reply-To: header
		if(!preg_match('/^Reply-To: /im', $headers))
			$headers .= "\n".'Reply-To: '.$from;

		// Return-Path: header --to process errors
		if(!preg_match('/^Return-Path: /im', $headers))
			$headers .= "\n".'Return-Path: '.$from;

		// Message-ID: header --helps to avoid spam filters
		if(!preg_match('/^Message-ID: /im', $headers))
			$headers .= "\n".'Message-ID: <'.time().'@'.$context['host_name'].'>';

  		// MIME-Version: header
		if(!preg_match('/^MIME-Version: /im', $headers))
			$headers .= "\n".'MIME-Version: 1.0';

		// ensure utf-8
		$message = utf8::from_unicode($message);

		// Content-Type: header
		if(!preg_match('/^Content-Type: /im', $headers))
			$headers .= "\n".'Content-Type: text/plain; charset=utf-8';

		// encoding rule
		if(!isset($context['mail_encoding']) || ($context['mail_encoding'] != '8bit'))
			$context['mail_encoding'] = 'base64';

		// encode the message for it transfer
		if($context['mail_encoding'] == 'base64') {

			// do the encoding
			$message = chunk_split(base64_encode($message));

			// Content-Transfer-Encoding: header
			if(!preg_match('/^Content-Transfer-Encoding: /im', $headers))
				$headers .= "\n".'Content-Transfer-Encoding: base64';

		// transmit native content
		} else {

			// Content-Transfer-Encoding: header
			if(!preg_match('/^Content-Transfer-Encoding: /im', $headers))
				$headers .= "\n".'Content-Transfer-Encoding: 8bit';

		}

		// X-Mailer: header --helps to avoid spam filters
		if(!preg_match('/^X-Mailer: /im', $headers))
			$headers .= "\n".'X-Mailer: PHP v'.phpversion();

		// strip leading spaces and newlines
		$headers = trim($headers);

		// debug mode
		if(isset($context['debug_mail']) && ($context['debug_mail'] == 'Y')) {
			$text = $headers."\n"
				.'To: '.(is_array($to) ? '<multiple addresses>' : $to)."\n"
				.'Subject: '.$subject."\n\n"
				.$message;

			Logger::remember($debug, 'Sending message by e-mail', $text, 'debug');
		}

		// make an array of recipients
		if(!is_array($to))
			$to = explode(',', $to);

		// process every recipient
		$posts = 0;
		foreach($to as $recipient) {

			// clean the provided string
			$recipient = trim(str_replace(array("\r\n", "\r", "\n", "\t"), ' ', $recipient));

			// extract the actual e-mail address -- Foo Bar <foo@bar.com> => foo@bar.com
			$tokens = explode(' ', $recipient);
			$actual_recipient = trim(str_replace(array('<', '>'), '', $tokens[count($tokens)-1]));

			// post in debug mode, to get messages, if any
			if(isset($context['debug_mail']) && ($context['debug_mail'] == 'Y') && mail($actual_recipient, $subject, $message, $headers))
				$posts++;

			// regular post
			elseif(!(isset($context['debug_mail']) && ($context['debug_mail'] == 'Y')) && @mail($actual_recipient, $subject, $message, $headers))
				$posts++;

			// an error has been encountered
			else
				Skin::error(sprintf(i18n::s('Error while sending the message to %s'), $recipient));

			// take care of time
			if(!($posts%50)) {

				// ensure enough execution time
				Safe::set_time_limit(30);

				// ease the pain of mail server
				Safe::sleep(1);
			}
		}

		// track last successful post
		include_once $context['path_to_root'].'shared/values.php';
		Values::set('mail.outbound', $subject.' ('.$posts.' recipients)');

		// return the number of actual posts
		return $posts;
	}

}

?>